@@ -21,7 +21,7 @@ from prompt_toolkit.key_bindings.emacs import emacs_bindings
 from prompt_toolkit.key_bindings.vi import vi_bindings
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.layout import Layout
-from prompt_toolkit.layout.menus import CompletionMenu
+from prompt_toolkit.layout.menus import CompletionsMenu
 from prompt_toolkit.layout.processors import BracketsMismatchProcessor
 from prompt_toolkit.layout.toolbars import CompletionsToolbar, ArgToolbar, SearchToolbar, ValidationToolbar, SystemToolbar
 from prompt_toolkit.layout.toolbars import Toolbar
@@ -204,6 +204,8 @@ class PythonLine(Line):
     """
     Custom `Line` class with some helper functions.
     """
+    _multiline_string_delims = re.compile('''[']{3}|["]{3}''')
+
     def __init__(self, always_multiline, *a, **kw):
         self.always_multiline = always_multiline
         super(PythonLine, self).__init__(*a, **kw)
@@ -215,10 +217,6 @@ class PythonLine(Line):
         #: newline.
         self.paste_mode = False
 
-        #: Boolean `multiline` flag. If True, [Enter] will always insert a
-        #: newline, and it is required to use [Meta+Enter] execute commands.
-        self.is_multiline = self.always_multiline
-
         # Code signatures. (This is set asynchronously after a timeout.)
         self.signatures = []
 
@@ -247,12 +245,30 @@ class PythonLine(Line):
                 for x in range(4):
                     insert_text(' ')
 
+    @property
+    def _ends_in_multiline_string(self):
+        """
+        ``True`` if we're inside a multiline string at the end of the text.
+        """
+        delims = self._multiline_string_delims.findall(self.text)
+        opening = None
+        for delim in delims:
+            if opening is None:
+                opening = delim
+            elif delim == opening:
+                opening = None
+        return bool(opening)
+
     @property
     def is_multiline(self):
         """
         Dynamically determine whether we're in multiline mode.
         """
-        if self.always_multiline or self.paste_mode or '\n' in self.text:
+        if any([
+                self.always_multiline,
+                self.paste_mode,
+                '\n' in self.text,
+                self._ends_in_multiline_string]):
             return True
 
         # If we just typed a colon, or still have open brackets, always insert a real newline.
@@ -467,6 +483,10 @@ class PythonCompleter(Completer):
                 # Issue #9: bad syntax causes completions() to fail in jedi.
                 # https://github.com/jonathanslenders/python-prompt-toolkit/issues/9
                 pass
+            except UnicodeDecodeError:
+                # Issue #43: UnicodeDecodeError on OpenBSD
+                # https://github.com/jonathanslenders/python-prompt-toolkit/issues/43
+                pass
             else:
                 for c in completions:
                     yield Completion(c.name_with_symbols, len(c.complete) - len(c.name_with_symbols),
@@ -499,7 +519,7 @@ class PythonCommandLineInterface(CommandLineInterface):
             min_height=7,
             lexer=PythonLexer,
             left_margin=PythonLeftMargin(),
-            menus=[CompletionMenu()] if autocompletion_style == AutoCompletionStyle.POPUP_MENU else [],
+            menus=[CompletionsMenu()] if autocompletion_style == AutoCompletionStyle.POPUP_MENU else [],
             bottom_toolbars=[
                 ArgToolbar(),
                 SignatureToolbar(),
