@@ -37,13 +37,14 @@ Example:
         print(tree)
 
 """
+from __future__ import unicode_literals
 from pygments.token import Token
 
 import six
 import re
 
 from .nodes import AnyNode, SequenceNode, RepeatNode, LiteralNode, VariableNode, EmptyNode
-from prompt_toolkit.code import Completion
+from prompt_toolkit.completion import Completion
 
 __all__ = (
     # Classes for defining the grammar.
@@ -78,9 +79,7 @@ class Rule(object):
         """
         return False
 
-    def parse(self, lexer_stream, allow_incomplete_trees=True):
-                                            # TODO: implement option: allow_incomplete_trees=True
-                                            # TODO: OR REMOVE THIS OPTION (I don't think we still need it.)
+    def parse(self, lexer_stream):
         """
         Yield a list of possible parse trees.
 
@@ -283,7 +282,7 @@ class Literal(Rule):
 
     def complete(self, text=''):
         if self.text.startswith(text):
-            yield Completion(self.text, self.text[len(text):])
+            yield Completion(self.text, -len(text))
 
 
 class Variable(Rule):
