@@ -12,7 +12,7 @@ from .utils import TokenList
 
 __all__ = (
     'ArgToolbar',
-    'CompletionToolbar',
+    'CompletionsToolbar',
     'SearchToolbar',
     'SystemToolbar',
     'TextToolbar',
@@ -22,7 +22,7 @@ __all__ = (
 
 class Toolbar(object):
     def __init__(self, token=None):
-        self.token = token or Token.Layout.Toolbar
+        self.token = token or Token.Toolbar
 
     def write(self, cli, screen):
         width = screen.size.columns
@@ -70,7 +70,7 @@ class ArgToolbar(Toolbar):
     A simple toolbar which shows the repeat 'arg'.
     """
     def __init__(self, token=None):
-        token = token or Token.Layout.Toolbar.Arg
+        token = token or Token.Toolbar.Arg
         super(ArgToolbar, self).__init__(token=token)
 
     def is_visible(self, cli):
@@ -79,8 +79,8 @@ class ArgToolbar(Toolbar):
 
     def get_tokens(self, cli, width):
         return [
-            (Token.Layout.Toolbar.Arg, 'Repeat: '),
-            (Token.Layout.Toolbar.Arg.Text, str(cli.input_processor.arg)),
+            (Token.Toolbar.Arg, 'Repeat: '),
+            (Token.Toolbar.Arg.Text, str(cli.input_processor.arg)),
         ]
 
 
@@ -89,7 +89,7 @@ class SystemToolbar(Toolbar):
     The system toolbar. Shows the '!'-prompt.
     """
     def __init__(self):
-        token = Token.Layout.Toolbar.System
+        token = Token.Toolbar.System
         super(SystemToolbar, self).__init__(token=token)
 
         # We use a nested single-line-no-wrap layout for this.
@@ -109,7 +109,7 @@ class SystemToolbar(Toolbar):
 
 class SearchToolbar(Toolbar):
     def __init__(self, token=None):
-        token = token or Token.Layout.Toolbar.Search
+        token = token or Token.Toolbar.Search
         super(SearchToolbar, self).__init__(token=token)
 
         class Prefix(Prompt):
@@ -148,17 +148,17 @@ class SearchToolbar(Toolbar):
         self.layout.write_content(cli, screen)
 
 
-class CompletionToolbar(Toolbar):
+class CompletionsToolbar(Toolbar):
     """
     Helper for drawing the completion menu 'wildmenu'-style.
     (Similar to Vim's wildmenu.)
     """
     def __init__(self, token=None):
-        token = token or Token.CompletionToolbar
-        super(CompletionToolbar, self).__init__(token=token)
+        token = token or Token.Toolbar.Completions
+        super(CompletionsToolbar, self).__init__(token=token)
 
     def is_visible(self, cli):
-        return super(CompletionToolbar, self).is_visible(cli) and \
+        return super(CompletionsToolbar, self).is_visible(cli) and \
             bool(cli.line.complete_state) and len(cli.line.complete_state.current_completions) >= 1
 
     def get_tokens(self, cli, width):
@@ -219,12 +219,12 @@ class ValidationToolbar(Toolbar):
     Toolbar for displaying validation errors.
     """
     def __init__(self, token=None):
-        token = token or Token.ValidationToolbar
+        token = token or Token.Toolbar.Validation
         super(ValidationToolbar, self).__init__(token=token)
 
     def is_visible(self, cli):
         return super(ValidationToolbar, self).is_visible(cli) and \
-                bool(cli.line.validation_error)
+            bool(cli.line.validation_error)
 
     def get_tokens(self, cli, width):
         if cli.line.validation_error:
